#version 120
#extension GL_EXT_gpu_shader3 : enable

uniform int fogMode;
uniform int dissolveMode;

uniform vec2 ensembleParamA;
uniform float ensembleParamB;
uniform vec2 ensembleParamC;

uniform float projInv11;
uniform float projInv15;

varying vec3 position;
varying vec4 color;

void FogEffect(){

    //float z = gl_FragCoord.z/ gl_FragCoord.w;

    const float near = 0;//gl_DepthRange.near;
    const float far = 1;//gl_DepthRange.far;

    float ndcPos_z = (2.0 * gl_FragCoord.z - near - far) / (far - near);
    float clip_z = ndcPos_z / gl_FragCoord.w;
    float clip_w = 1.0 / gl_FragCoord.w;
    //float p_z = -clip_w;
    float p_w = projInv11 * clip_z + projInv15 * clip_w;
    //float z = -p_z * p_w;
    float z = clip_w * p_w;
	
    vec4 fogColor    = gl_Fog.color;
    fogColor[3] = gl_FragColor[3];
    float fogDensity = gl_Fog.density;

    float fogFactor = 0.0;
    if (fogMode == 3){
	fogFactor = (gl_Fog.end - z) / (gl_Fog.end - gl_Fog.start);
    }
    else if (fogMode == 2){
	float LOG2 = 1.442695;
	fogFactor = exp2( -gl_Fog.density * z * LOG2 );
    }
    else {
	float LOG2 = 1.442695;
	fogFactor = exp2( -gl_Fog.density * gl_Fog.density * z * z * LOG2 );
    }
    
    fogFactor = clamp(fogFactor, 0.0, 1.0);
 
    gl_FragColor = mix(fogColor, gl_FragColor, fogFactor);
}

void main (void)
{
    float alpha = color[3];//gl_FrontLightProduct[0].ambient[3];

    if (dissolveMode >= 1){
	float bound = 0.015;
	if (dissolveMode == 2) bound = 1;
	
	if (alpha <= bound){
	    float crit = fract(sin(dot(gl_FragCoord.xy + position.xz + ensembleParamA ,
				       /*vec2(12.9898,78.233)*/ensembleParamC)) * ensembleParamB) * bound;
	    if (alpha <= crit) discard;
	    else alpha = bound;
	}
    }

    if (alpha > 1.0) alpha = 1.0;
    
    gl_FragColor = color;
    gl_FragColor[3] = alpha;

    if (fogMode != 0) FogEffect();
}
