
#version 120
#extension GL_EXT_gpu_shader3 : enable

uniform sampler2D textureImage;
uniform int textureMode;
uniform int fogMode;
uniform int useTexAlpha;
uniform int dissolveMode;

varying float diffuseTotal;
varying float specularTotal;

varying vec3 position;

uniform vec2 ensembleParamA;
uniform float ensembleParamB;
uniform vec2 ensembleParamC;

uniform float projInv11;
uniform float projInv15;

void FogEffect(){

    //float z = gl_FragCoord.z/ gl_FragCoord.w;
    
    const float near = 0;//gl_DepthRange.near;
    const float far = 1;//gl_DepthRange.far;

    float ndcPos_z = (2.0 * gl_FragCoord.z - near - far) / (far - near);
    float clip_z = ndcPos_z / gl_FragCoord.w;
    float clip_w = 1.0 / gl_FragCoord.w;
    //float p_z = -clip_w;
    float p_w = projInv11 * clip_z + projInv15 * clip_w;
    //float z = -p_z * p_w;
    float z = clip_w * p_w;
    
    vec4 fogColor    = gl_Fog.color;
    fogColor[3] = gl_FragColor[3];
    float fogDensity = gl_Fog.density;

    float fogFactor = 0.0;
    if (fogMode == 3){
	fogFactor = (gl_Fog.end - z) / (gl_Fog.end - gl_Fog.start);
    }
    else if (fogMode == 2){
	float LOG2 = 1.442695;
	fogFactor = exp2( -gl_Fog.density * z * LOG2 );
    }
    else {
	float LOG2 = 1.442695;
	fogFactor = exp2( -gl_Fog.density * gl_Fog.density * z * z * LOG2 );
    }
    
    fogFactor = clamp(fogFactor, 0.0, 1.0);
 
    gl_FragColor = mix(fogColor, gl_FragColor, fogFactor);
}

void main (void)
{
    vec4 texColor = vec4(1.0,1.0,1.0,1.0); 
    if (textureMode == 1)
	texColor = texture2D(textureImage, gl_TexCoord[0].st);

    float alpha = gl_FrontLightProduct[0].ambient[3];
    if (useTexAlpha == 1){
	alpha *= texColor[3];
	if (alpha == 0) discard;
    }

    if (dissolveMode >= 1){
	float bound = 0.015;
	if (dissolveMode == 2) bound = 1;
	
	if (alpha <= bound){
	    float crit = 
		fract(sin(dot(position.xy + ensembleParamA ,
			      ensembleParamC)) * ensembleParamB) * bound;
	    
	    if (alpha <= crit) discard;
	    else alpha = bound;
	}
    }

    if (alpha > 1.0) alpha = 1.0;
    
    gl_FragColor = gl_FrontLightProduct[0].ambient + 
	gl_FrontMaterial.diffuse * diffuseTotal +
	gl_FrontMaterial.specular * specularTotal;
    gl_FragColor *= texColor;
    gl_FragColor[3] = alpha;

    if (fogMode != 0) FogEffect();
}
