// Scatter.geom

// SetParameter GL_GEOMETRY_INPUT_TYPE_EXT=GL_POINTS
// SetParameter GL_GEOMETRY_OUTPUT_TYPE_EXT=GL_TRIANGLE_STRIP
// SetParameter GL_GEOMETRY_VERTICES_OUT_EXT=4

#version 130
#extension GL_EXT_geometry_shader4 : enable
#extension GL_EXT_gpu_shader4 : enable

uniform float aspectAngleFactor; 

in vec3 positionIn[];
in int  subIndexIn[];
in float pointSizeIn[];
in vec3 normalIn[];
in vec4 diffuseAndAmbientIn[];

out vec3 position;
out vec3 normal;
out vec4 diffuseAndAmbient;

void main(void){

    vec4 offset[4] = vec4[]
	(vec4(-1, -1, 0,0), vec4(1, -1, 0,0), vec4(-1, 1, 0,0), vec4(1, 1, 0,0));

    int subIndex = subIndexIn[0];
    int atrasGrid = subIndex / 256;
    if (atrasGrid == 0) atrasGrid = 1;
    subIndex = subIndex % 256;

    float atrasWidth = 1.0 / atrasGrid;
    float atrasHeight = 1.0 / atrasGrid;

    vec4 texCoord[4];
    int offsetX = subIndex % atrasGrid;
    int offsetY = subIndex / atrasGrid;
    texCoord[0] = vec4(offsetX * atrasWidth , offsetY * atrasHeight, 0, 0);
    texCoord[1] = vec4((offsetX + 1) * atrasWidth, offsetY * atrasHeight, 0, 0);
    texCoord[2] = vec4(offsetX * atrasWidth, (offsetY + 1) * atrasHeight, 0, 0);
    texCoord[3] = vec4((offsetX + 1) * atrasWidth, (offsetY + 1) * atrasHeight, 0, 0);
    
    /*
    vec4 texCoord[4] = vec4[]
	(vec4(0,0,0,0), vec4(1,0,0,0), vec4(0,1,0,0), vec4(1,1,0, 0)); 
    */
    
    float psize = pointSizeIn[0] * aspectAngleFactor;

    for(int i = 0; i < gl_VerticesIn; i++){
	diffuseAndAmbient = diffuseAndAmbientIn[i];
	normal = normalIn[i];
	
	gl_FrontColor = gl_FrontColorIn[i];
	gl_TexCoord[0] = texCoord[0];
	gl_Position = gl_PositionIn[i] + offset[0] * psize;
	position = positionIn[i] + vec3(offset[0]*psize);
	EmitVertex();
	gl_TexCoord[0] = texCoord[1];
	gl_Position = gl_PositionIn[i] + offset[1] * psize;
	position = positionIn[i] + vec3(offset[1]*psize);
	EmitVertex();
	gl_TexCoord[0] = texCoord[2];
	gl_Position = gl_PositionIn[i] + offset[2] * psize;
	position = positionIn[i] + vec3(offset[2]*psize);
	EmitVertex();
	gl_TexCoord[0] = texCoord[3];
	gl_Position = gl_PositionIn[i] + offset[3] * psize;
	position = positionIn[i] + vec3(offset[3]*psize);
	EmitVertex();
    }
    
    EndPrimitive();
}
